package lopsil.device;

import lopsil.*;
import gpslib4j.*; 

/**
 * The Location class encapsulates the Locations a LocationDevice can have.
 * Currently, Window devices use Strings as Locations and GPS devices
 * use Positions as Locations.
 * 
 * @author Billy Rickey
 * @author Jay Ligatti
 * @author Nalin Saigal
 * 
 */
public class Location {

	/** Instance Fields */
	private Object _geoLocation;
	private String _windowLocation;
	private Position _gpsLocation;
	private boolean _isGPS, _isWindow;
	private volatile long _updateTime;
	private static Policy _securityPolicy;
	
	/**
	 * Construct a newly allocated Location.
	 */
	public Location(){
		_geoLocation = new Object();
		_updateTime = System.currentTimeMillis();
	}
	
	/**
	 * Construct a newly allocated Location with a specified Position.
	 * @param p the Position representing the Location of a GPS LocationDevice
	 */
	public Location(Position p) {
		_gpsLocation = p;
		_updateTime = System.currentTimeMillis();
		_isGPS = true;
	}

	/**
	 * Construct a newly allocated Location with a specified String
	 * @param s the String representing the Location of a Window LocationDevice
	 * and allocates to it the coordinates defined in file MapLocationToCoords.java
	 */
	public Location(String s){
		_gpsLocation = lopsil.device.MapLocationToCoords.
								returnCoords(s.toUpperCase());
		_windowLocation = s;
		_updateTime = System.currentTimeMillis();
		_isWindow = true;
	}

	/**
	 * Constructs a newly allocated Location with a specified String
	 * and allocates it the coordinates, also specified by the user.
	 */
	public Location(String s, Position p)
	{
		_gpsLocation = p;
		_windowLocation = s;
		_updateTime = System.currentTimeMillis();
		_isWindow = true;
	}
	
	/**
	 * Returns the geographic location.
	 * @return the location
	 * @deprecated Use <code>getWindowLocation()</code> or <code>getGPSLocation()</code>
	 */
	public synchronized Object getGeoLocation(){return _geoLocation;}
	
	/**
	 * Returns the specific geographic location of a Window device.
	 * @return the specific location of a Window device
	 */
	public synchronized String getWindowLocation(){
		if(_windowLocation != null) {
			try {
				checkLocation("window");
			} catch (LopsilException e) {
				System.out.println(e);
				System.exit(1);
			}
		}
		return _windowLocation;
	}
	
	/**
	 * Returns the specific geographic location of a GPS device.
	 * @return the specific location of a GPS device
	 */
	public synchronized Position getGPSLocation(){
		if (_isWindow)
			return _gpsLocation;
		if(_gpsLocation != null) {
			try{
				checkLocation("gps");
			} catch(LopsilException e) {
				System.out.println(e);
				System.exit(1);
			}
		}
		return _gpsLocation;
	}
	
	/**
	 * Sets the specific geographic location of a Window device.
	 * @param windowLoc the specific location of a Window device
	 */
	public synchronized void setWindowLocation(String windowLoc)
	{
		_windowLocation = windowLoc;
		_updateTime = System.currentTimeMillis();
		_gpsLocation = lopsil.device.MapLocationToCoords.returnCoords(windowLoc.toUpperCase());
		_isWindow = true;
		if (_securityPolicy == null)
		{ /* do nothing */}
		else /* update the policy */
			_securityPolicy.onLocationUpdate();
	}
	
	/**
	 * Sets the specific geographic location of a GPS device.
	 * @param gpsLoc the specific location of a GPS device
	 */
	public synchronized void setGPSLocation(Position gpsLoc) {
		_gpsLocation = gpsLoc;
		_updateTime = System.currentTimeMillis();
		_isGPS = true;
		if (_securityPolicy == null)
		{ /* do nothing */}
		else /* update the policy */
			_securityPolicy.onLocationUpdate();
	}
	
	/**
	 * Sets the geographic location.
	 * @param loc the location to be set
	 * @deprecated use <code>setWindowLocation()</code> or <code>setGPSLocation</code>
	 */
	public synchronized void setGeoLocation(Object loc){
		_updateTime = System.currentTimeMillis();
		_geoLocation = loc;
		if (_securityPolicy == null)
		{ /* do nothing */}
		else /* update the policy */
			_securityPolicy.onLocationUpdate();
	}
	
	/**
	 * Returns the time of the most recent update.
	 * @return the time of the most recent update
	 */
	public long getLastTimeOfUpdate(){return this._updateTime;}
	
	/**
	 * Checks to see if a Location has been updated within a specific time interval
	 * 
	 * @param time
	 * @param measurement
	 * @return true if the Location has been updated within the time interval, and false otherwise
	 */
	public boolean isUpdatedWithin(double time, int measurement) {
		switch(measurement) {
			case LopsilUtil.SECONDS:
				time *= 1000.0;
				break;
			case LopsilUtil.MINUTES:
				time *= 60000.0;
				break;
			case LopsilUtil.HOURS:
				time *= 3600000.0;
				break;
		}		
		return System.currentTimeMillis() - _updateTime < time;
	}


	public boolean isWindowDevice() { return _isWindow; }

	/**
	 * Returns the string representation of the Location.
	 */
	public String toString(){
		if (_isGPS)
			return _gpsLocation.toString();
		else if (_isWindow)
			return _windowLocation.toString();
		else
			return "";
	}
	
	public static void setPolicy(Policy p) {
		_securityPolicy = p;
	}

	/**
	 *  Returns the distance of the location from a location l,
	 *  in the units specified by the user
	 */
	public double distance(Location l, int units)
	{
		double multiplier = Units.getMultiplier(units);
		double distance = this._gpsLocation.getDistanceFrom(l._gpsLocation);
		return (distance / multiplier);
	}

	/**
	 * Returns true or false based on whether the current location 
	 * is within the region defined by regionLocs.
	 */
	public boolean inRegion(Location[] regionLocs)
	{
		PositionRadians minLat = regionLocs[0]._gpsLocation.getLatitude();
		PositionRadians minLon = regionLocs[0]._gpsLocation.getLongitude();
		PositionRadians maxLat = regionLocs[0]._gpsLocation.getLatitude();
		PositionRadians maxLon = regionLocs[0]._gpsLocation.getLongitude();
		PositionRadians compareWith;
		/* Get minimum Latitude */
		for (int i = 0; i < regionLocs.length; i++)
		{
			compareWith = regionLocs[i]._gpsLocation.getLatitude();
			if (compareWith.smallerThan(minLat))
				minLat = compareWith;
		}
		/* Get minimum Longitude */
		for (int i = 0; i < regionLocs.length; i++)
		{
			compareWith = regionLocs[i]._gpsLocation.getLongitude();
			if (compareWith.smallerThan(minLon))
				minLon = compareWith;
		}
		/* Get maximum Latitude */
		for (int i = 0; i < regionLocs.length; i++)
		{
			compareWith = regionLocs[i]._gpsLocation.getLatitude();
			if (compareWith.greaterThan(maxLat))
				maxLat = compareWith;
		}
		/* Get maximum Longitude */
		for (int i = 0; i < regionLocs.length; i++)
		{
			compareWith = regionLocs[i]._gpsLocation.getLongitude();
			if (compareWith.greaterThan(maxLon))
				maxLon = compareWith;
		}
		PositionRadians thisLat = this._gpsLocation.getLatitude();
		PositionRadians thisLon = this._gpsLocation.getLongitude();
		if ((thisLat.smallerThan(maxLat) || thisLat.equals(maxLat)) &&
			(thisLat.greaterThan(minLat) || thisLat.equals(minLat)) &&
			(thisLon.smallerThan(maxLon) || thisLon.equals(maxLon)) &&
			(thisLon.greaterThan(minLon) || thisLon.equals(minLon)))
			return true;
		else
			return false;
	}
	/**
	 * Checks to make sure that the Location being asked for is the correct type.
	 * @param loc a String representation of the type of Location
	 * @throws LopsilException
	 */
	private void checkLocation(String loc) throws LopsilException
	{
		if((loc.equals("window") && !_isWindow) || (loc.equals("gps") && !_isGPS))
			throw new LopsilException("Location not set or incorrect usage!");
	}	
} // end Location