package lopsil.device;

import javax.swing.JOptionPane;

import java.io.*;
import javax.comm.*;
import gpslib4j.*;
import gpslib4j.Garmin.*;

/**
 * LopsilGPS is a default GPS LocationDevice that is provided with the LoPSiL package.
 * It uses a Garmin GPS device along with the Java Communication API.
 * 
 * @author Billy Rickey
 * @author Jay Ligatti
 * @author Nalin Saigal
 * 
 * @see GPS
 * @see GarminGPS
 */
public class LopsilGPS extends LocationDevice  {
	
	private GPS gps;		
	private BufferedInputStream input;
	private BufferedOutputStream output;
	private String _comPort;
	
	private static Location _location;
	//private Location[] _prevLocations;
	
	/** The number of previous positions to maintain */
	private static final int numPrev = 3;
	
	/** LopsilGPS representation for a Garmin GPS */
	public static final int GARMIN = 0;

	private static final double _granLength = 15.0;
	private static final int _granScale = Units.METERS;	
	
	/**
	 * Create a newly allocated LopsilGPS at a specified communication port.
	 * @param comPort the number of the COM port to be used
	 */
	public LopsilGPS(int type){
		int comPort = 4;
			_location = new Location();
			_comPort = "COM" + comPort;

			this._gps = true;
			this._gran = new LocationGranularityAssumption(_granLength, _granScale);

			CommPort port;

			// Try connecting to communication port
			try
			{
				port = CommPortIdentifier.getPortIdentifier(_comPort).open("LoPSiL GPS", 3000);
			}
			catch (NoSuchPortException e)
			{
				//System.out.println(_comPort + " not found!");
				JOptionPane.showConfirmDialog(null, _comPort + " not found!", "Comm Error", JOptionPane.PLAIN_MESSAGE);
				return;
			}
			catch (PortInUseException e)
			{
				JOptionPane.showConfirmDialog(null, "Port already in use by " + e.currentOwner,
						"Comm Error", JOptionPane.PLAIN_MESSAGE);
				//System.out.println("Port already in use by " + e.currentOwner);
				return;
			}

			try
			{
				input = new BufferedInputStream(port.getInputStream());
				output = new BufferedOutputStream(port.getOutputStream());
			}
			catch (IOException e)
			{
				System.out.println("Error opening " + _comPort);
				return;
			}
		// Start GPS transmission
		switch (type)
		{
			case GARMIN:
				gps = new GarminGPS(input, output, _location, numPrev);
				gps.setAutoTransmit(true);
				this._active = true;
				break;
		}
		
	}
	
	/**
	 * Returns an array consisting of the previous GPS Locations
	 * @return an array consisting of the previous GPS Locations
	 */
	public Location[] getPreviousLocations(){
		return gps.getPrevLocations();
	}
	
	public boolean isActive() {
		return gps.isActive();
	}
	
	/**
	 * Returns true if the GPS device has been updated.
	 * @see GPS
	 */
	public synchronized boolean isUpdated() {
		if(gps != null)
			return gps.isUpdated();
		return false;
	}
	
	/**
	 * Sets the GPS device status as 'not updated'
	 * @see GPS
	 */
	public synchronized void setNotUpdated() {
		if(gps != null)
			gps.setNotUpdated();
	}
	
	/** 
	 * Sets the GPS device status as 'updated'
	 * @see GPS
	 */
	public synchronized void setUpdated() {
		if(gps != null)
			gps.setUpdated();
	}
	
	/**
	 * Returns the Location of the LocationDevice
	 * @return the Location object of the LocationDevice
	 */
	public Location getLocation(){return this._location;}
	
	/**
	 * Sets the Location of the LocationDevice
	 * @param l the Location object to be set
	 */
	public void setLocation(Location l){this._location.setGPSLocation(l.getGPSLocation());}

} // end LopsilGPS